// -*- C++ -*-
// CLASSDOC OFF
// $Id: Normal3D.h,v 1.7 1999/04/06 17:13:32 evc Exp $
// ---------------------------------------------------------------------------
// CLASSDOC ON
//
// Hep 3D Normal class
//
// This file is part of Geant4 (simulation toolkit for HEP).
//
// Author: Evgeni Chernyaev <Evgueni.Tcherniaev@cern.ch>
//
// History:
// 13.09.96 E.Chernyaev - initial version
 
#ifndef HepNormal3D_hh
#define HepNormal3D_hh

#include "CLHEP/config/CLHEP.h"
#include "CLHEP/Vector/ThreeVector.h"

class HepTransform3D;

class HepNormal3D : public Hep3Vector {
 public:
  // Constructor
  HepNormal3D(HepDouble x=0, HepDouble y=0, HepDouble z=0)
    : Hep3Vector(x, y, z) {}

  // Copy Constructor
  HepNormal3D(const Hep3Vector &v) : Hep3Vector(v) {}

  // Destructor
  ~HepNormal3D() {};

  // Assignment
  HepNormal3D& operator=(const HepNormal3D &v) {
    setX(v.x()); setY(v.y()); setZ(v.z()); return *this;
  }

  // Assignment of Hep3Vector and classes derived from it (Point3D, Vector3D)
  HepNormal3D& operator=(const Hep3Vector &v) {
    setX(v.x()); setY(v.y()); setZ(v.z()); return *this;
  }

  // Transformation
  HepNormal3D& transform(const HepTransform3D &m);
};

#include "CLHEP/Geometry/Transform3D.h"

inline HepNormal3D& HepNormal3D::transform(const HepTransform3D &m) {
  return *this = m * (*this);
}

#endif /* HepNormal3D_hh */
