// -*- C++ -*-
// CLASSDOC OFF
// $Id: Vector3D.h,v 1.7 1999/04/06 17:13:34 evc Exp $
// ---------------------------------------------------------------------------
// CLASSDOC ON
//
// Hep geometrical 3D Vector class
//
// This file is part of Geant4 (simulation toolkit for HEP).
//
// Author: Evgeni Chernyaev <Evgueni.Tcherniaev@cern.ch>
//
// History:
// 13.09.96 E.Chernyaev - initial version
 
#ifndef HepVector3D_hh
#define HepVector3D_hh

#include "CLHEP/config/CLHEP.h"
#include "CLHEP/Vector/ThreeVector.h"

class HepTransform3D;

class HepVector3D : public Hep3Vector {
 public:
  // Constructor
  HepVector3D(HepDouble x=0, HepDouble y=0, HepDouble z=0)
    : Hep3Vector(x, y, z) {}

  // Copy Constructor
  HepVector3D(const Hep3Vector &v) : Hep3Vector(v) {}

  // Destructor
  ~HepVector3D() {};

  // Assignment
  HepVector3D & operator=(const HepVector3D &v) {
    setX(v.x()); setY(v.y()); setZ(v.z()); return *this;
  }

  // Assignment of Hep3Vector and classes derived from it (Point3D, Normal3D)
  HepVector3D& operator=(const Hep3Vector &v) {
    setX(v.x()); setY(v.y()); setZ(v.z()); return *this;
  }

  // Transformation
  HepVector3D& transform(const HepTransform3D &m);
};

#include "CLHEP/Geometry/Transform3D.h"

inline HepVector3D& HepVector3D::transform(const HepTransform3D &m) {
  return *this = m * (*this);
}

#endif /* HepVector3D_hh */
