// $Id: NonRandomEngine.h,v 1.2 2000/04/25 14:50:20 evc Exp $
// -*- C++ -*-
//
// -----------------------------------------------------------------------
//                             HEP Random
//                        --- NonRandomEngine ---
//                          class header file
// -----------------------------------------------------------------------

// This class is present EXCLUSIVELY as a means to test distributions (and
// other programs that depend on random numbers) by feeding them a stream
// of "randoms" that the testing program supplies explicitly.
//
// The testing program calls setNextRandom (HepDouble) to setup the next
// value to be produced when flat() is done.  
//
// To protect against accidental use of this NON-RANDOM engine as a random
// engine, if setNextRandom () is never called, all attempts to generate
// a random will fail and exit.

// =======================================================================
// Mark Fischler  - Created: 9/30/99
// =======================================================================

#ifndef NonRandomEngine_h
#define NonRandomEngine_h 1

#include "CLHEP/Random/RandomEngine.h"

class NonRandomEngine : public HepRandomEngine {

public:

  NonRandomEngine();
  virtual ~NonRandomEngine();
  // Constructors and destructor

  void setNextRandom     (HepDouble r);
	// Preset the next random to be delivered
  void setRandomSequence (HepDouble *s, HepInt n);
	// Establish a sequence of n next randoms; 
	// replaces setNextRandom n times.
  void setRandomInterval (HepDouble x);
	// Establish that if there is no sequence active each 
	// random should be bumped by this interval (mod 1) compared 
	// to the last.  x should be between 0 and 1.

  HepDouble flat();
  // It returns the previously established setNextRandom and bumps that up
  // by the non-zero randomInterval supplied.  Thus repeated calls to flat()
  // generate an evenly spaced sequence (mod 1).

  void flatArray (const HepInt size, HepDouble* vect);
  // Fills the array "vect" of specified size with flat random values.

private:

  HepBoolean nextHasBeenSet;
  HepBoolean sequenceHasBeenSet;
  HepBoolean intervalHasBeenSet;
  HepDouble  nextRandom;
  HepDouble *sequence;
  HepInt     nSequence;
  HepDouble  randomInterval;

  // The following are necessary to fill virtual methods but should never 
  // be used:

  virtual void setSeed(long seed, HepInt) {};
  virtual void setSeeds(const long * seeds, HepInt) {};
  virtual void saveStatus( const char filename[] = "Config.conf") const {};
  virtual void restoreStatus( const char filename[] = "Config.conf" ) {};
  virtual void showStatus() const {};

};

#endif
