// $Id: RandChiSquare.h,v 1.4 2000/04/17 22:59:54 mf Exp $
// -*- C++ -*-
//
// -----------------------------------------------------------------------
//                             HEP Random
//                         --- RandChiSquare ---
//                          class header file
// -----------------------------------------------------------------------

// Class defining methods for shooting Chi^2 distributed random values,
// given a number of degrees of freedom a (default=1.0).
// Default values are used for operator()().

// Valid values of a satisfy a > 1. When invalid values are presented,
// the code silently returns -1.0.

// =======================================================================
// John Marraffino - Created: 12th May 1998  Based on the C-Rand package
//                   by Ernst Stadlober and Franz Niederl of the Technical
//                   University of Graz, Austria.
// Gabriele Cosmo  - Removed useless methods and data: 5th Jan 1999
// =======================================================================

#ifndef RandChiSquare_h
#define RandChiSquare_h 1

#include "CLHEP/Random/Random.h"

class RandChiSquare : public HepRandom {

public:

  inline RandChiSquare ( HepRandomEngine& anEngine, HepDouble a=1 );
  inline RandChiSquare ( HepRandomEngine* anEngine, HepDouble a=1 );
  // These constructors should be used to instantiate a RandChiSquare
  // distribution object defining a local engine for it.
  // The static generator will be skipped using the non-static methods
  // defined below.
  // If the engine is passed by pointer the corresponding engine object
  // will be deleted by the RandChiSquare destructor.
  // If the engine is passed by reference the corresponding engine object
  // will not be deleted by the RandChiSquare destructor.

  virtual ~RandChiSquare();
  // Destructor

  // Static methods to shoot random values using the static generator

  static inline HepDouble shoot();

  static HepDouble shoot( HepDouble a );

  static void shootArray ( const HepInt size, HepDouble* vect,
                            HepDouble a=1.0 );

  //  Static methods to shoot random values using a given engine
  //  by-passing the static generator.

  static inline HepDouble shoot( HepRandomEngine* anEngine );

  static HepDouble shoot( HepRandomEngine* anEngine, 
                                  HepDouble a );

  static void shootArray ( HepRandomEngine* anEngine, const HepInt size,
                            HepDouble* vect, HepDouble a=1.0 );

  //  Methods using the localEngine to shoot random values, by-passing
  //  the static generator.

  inline HepDouble fire();

  HepDouble fire( HepDouble a );
  
  void fireArray ( const HepInt size, HepDouble* vect);
  void fireArray ( const HepInt size, HepDouble* vect,
                   HepDouble a );
  inline HepDouble operator()();
  inline HepDouble operator()( HepDouble a );

private:

  // Private copy constructor. Defining it here disallows use.
  RandChiSquare(const RandChiSquare& d);

  static HepDouble genChiSquare( HepRandomEngine *anEngine, HepDouble a );

  HepRandomEngine* localEngine;
  HepBoolean deleteEngine;
  const HepDouble defaultA;

};

#include "CLHEP/Random/RandChiSquare.icc"

#endif
