// $Id: RandStudentT.h,v 1.4 2000/04/17 23:00:21 mf Exp $
// -*- C++ -*-
//
// -----------------------------------------------------------------------
//                             HEP Random
//                         --- RandStudentT ---
//                          class header file
// -----------------------------------------------------------------------

// Class defining methods for shooting Student's t- distributed random 
// values, given a number of degrees of freedom a (default=1.0).
// Default values are used for operator()().

// Valid input values are a > 0.  When invalid values are presented, the
// code silently returns HUGE_VAL from <math.h> which is the same as
// MAXDOUBLE in <values.h> on systems where the latter exists.

// =======================================================================
// John Marraffino - Created: Based on the C-Rand package
//                   by Ernst Stadlober and Franz Niederl of the Technical
//                   University of Graz, Austria : 12th May 1998
//                 - Removed <values.h> because that won't work
//                   on NT : 26th Jun 1998
// Gabriele Cosmo  - Fixed minor bug on inline definition for shoot()
//                   methods. Created .icc file : 20th Aug 1998
//                 - Removed useless methods and data: 5th Jan 1999
// =======================================================================

#ifndef RandStudentT_h
#define RandStudentT_h 1

#include "CLHEP/Random/Random.h"

class RandStudentT : public HepRandom {

public:

  inline RandStudentT ( HepRandomEngine& anEngine, HepDouble a=1.0 );
  inline RandStudentT ( HepRandomEngine* anEngine, HepDouble a=1.0 );
  // These constructors should be used to instantiate a RandStudentT
  // distribution object defining a local engine for it.
  // The static generator will be skipped using the non-static methods
  // defined below.
  // If the engine is passed by pointer the corresponding engine object
  // will be deleted by the RandStudentT destructor.
  // If the engine is passed by reference the corresponding engine object
  // will not be deleted by the RandStudentT destructor.

  virtual ~RandStudentT();
  // Destructor

  // Static methods to shoot random values using the static generator

  static  inline HepDouble shoot();

  static  HepDouble shoot( HepDouble a );

  static  void shootArray ( const HepInt size, HepDouble* vect,
                            HepDouble a=1.0 );

  //  Static methods to shoot random values using a given engine
  //  by-passing the static generator.

  static  inline HepDouble shoot( HepRandomEngine* anEngine );

  static  HepDouble shoot( HepRandomEngine* anEngine, 
                           HepDouble a );

  static  void shootArray ( HepRandomEngine* anEngine, const HepInt size,
                            HepDouble* vect, HepDouble a=1.0 );

  //  Methods using the localEngine to shoot random values, by-passing
  //  the static generator.

  inline HepDouble fire();

  HepDouble fire( HepDouble a );
  
  void fireArray ( const HepInt size, HepDouble* vect );
  void fireArray ( const HepInt size, HepDouble* vect, HepDouble a );
  HepDouble operator()();
  HepDouble operator()( HepDouble a );

private:

  // Private copy constructor. Defining it here disallows use.
  RandStudentT(const RandStudentT& d);

  HepRandomEngine* localEngine;
  HepBoolean deleteEngine;
  const HepDouble defaultA;

};

#include "CLHEP/Random/RandStudentT.icc"

#endif
